@@ -2,7 +2,7 @@
  *
  * Translate algorithm identifiers (or names) to algorithm objects.
  *
- * $Id: algorithms.c,v 1.27 2001/10/15 12:38:18 nisse Exp $ */
+ * $Id: algorithms.c,v 1.31 2002/03/14 09:44:59 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -29,7 +29,7 @@
 #include "compress.h"
 #include "crypto.h"
 #include "dsa.h"
-#include "rsa.h"
+#include "publickey_crypto.h"
 #include "xalloc.h"
 
 #include "lsh_argp.h"
@@ -58,22 +58,25 @@ all_symmetric_algorithms()
 		    ATOM_SERPENT_CBC_LOCAL, &crypto_serpent256_cbc_algorithm,
 		    ATOM_3DES_CBC, &crypto_des3_cbc_algorithm,
 		    ATOM_CAST128_CBC, &crypto_cast128_cbc_algorithm,
-		    ATOM_HMAC_SHA1, make_hmac_algorithm(&sha1_algorithm),
-		    ATOM_HMAC_MD5, make_hmac_algorithm(&md5_algorithm),
+		    ATOM_HMAC_SHA1,
+		      make_hmac_algorithm(&crypto_sha1_algorithm),
+		    ATOM_HMAC_MD5,
+		      make_hmac_algorithm(&crypto_md5_algorithm),
 #if WITH_ZLIB
 		    ATOM_ZLIB, make_zlib(),
 #endif
 		    -1);
 }
 
-/* This is used for spki operations, and should therefore use spki names. */
+/* This is used for spki operations, and should therefore use spki names.
+ * Note that md5 signatures are not currently supported. */
 struct alist *
 all_signature_algorithms(struct randomness *r)
 {
   return make_alist(3,
 		    ATOM_DSA, make_dsa_algorithm(r),
+		    ATOM_RSA_PKCS1, &rsa_sha1_algorithm.super,
 		    ATOM_RSA_PKCS1_SHA1, &rsa_sha1_algorithm.super,
-		    ATOM_RSA_PKCS1_MD5, &rsa_md5_algorithm.super,
 		    -1);
 }
 
@@ -86,7 +89,8 @@ filter_algorithms_l(struct alist *algorithms, unsigned n, ...);
 static struct int_list *
 default_crypto_algorithms(struct alist *algorithms)
 {
-  return filter_algorithms_l(algorithms, 3,
+  return filter_algorithms_l(algorithms, 4,
+			     ATOM_AES256_CBC,
 			     ATOM_3DES_CBC,
 			     ATOM_BLOWFISH_CBC,
 			     ATOM_ARCFOUR, -1);
@@ -98,11 +102,11 @@ static struct int_list *
 all_crypto_algorithms(struct alist *algorithms)
 {
   return filter_algorithms_l(algorithms, 8,
+                             ATOM_AES256_CBC,
 			     ATOM_3DES_CBC,
 			     ATOM_TWOFISH_CBC, 			   
 			     ATOM_CAST128_CBC,
 			     ATOM_SERPENT_CBC_LOCAL,
-                             ATOM_AES256_CBC,
 			     ATOM_RIJNDAEL_CBC_LOCAL,
 			     ATOM_BLOWFISH_CBC,
 			     ATOM_ARCFOUR, -1);
