@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.106 2001/10/29 18:36:57 nisse Exp $
+ * $Id: channel.c,v 1.109 2002/03/14 15:35:08 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -393,7 +393,7 @@ register_channel(UINT32 local_channel_number,
   if (take_into_use)
     use_channel(channel->connection, local_channel_number);
   
-  REMEMBER_RESOURCE(channel->connection->resources,
+  remember_resource(channel->connection->resources,
 		    &channel->resources->super);
 }
 
@@ -1589,8 +1589,40 @@ DEFINE_PACKET_HANDLER(static, channel_failure_handler,
     PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_FAILURE message.");
 }
 
+static void
+do_channels_after_keyexchange(struct command_continuation *s UNUSED,
+			      struct lsh_object *x);
 
-void init_connection_service(struct ssh_connection *connection)
+static struct command_continuation
+channels_after_keyexchange =
+  { STATIC_HEADER, do_channels_after_keyexchange };
+    
+static void
+do_channels_after_keyexchange(struct command_continuation *s UNUSED,
+			      struct lsh_object *x)
+{
+  CAST(ssh_connection, connection, x);
+  struct channel_table *table = connection->table;
+  UINT32 i;
+  
+  /* Iterate over all channels, and call CHANNEL_SEND_ADJUST to get
+   * them started again. */
+
+  for (i = 0; i<table->used_channels; i++)
+    {
+      struct ssh_channel *channel = lookup_channel(table, i);
+      if (channel
+	  && !(channel->flags & (CHANNEL_SENT_CLOSE | CHANNEL_SENT_EOF))
+	  && channel->send_adjust)
+	CHANNEL_SEND_ADJUST(channel, 0);
+    }
+
+  /* Reinstall hook */
+  connection_after_keyexchange(connection, &channels_after_keyexchange);
+}
+
+void
+init_connection_service(struct ssh_connection *connection)
 {
   struct channel_table *table = make_channel_table();
   
@@ -1598,6 +1630,12 @@ void init_connection_service(struct ssh_connection *connection)
   
   connection->table = table;
 
+  /* Cancel handshake timeout */
+  connection_clear_timeout(connection);
+
+  /* Unfreeze channels after key exchange */
+  connection_after_keyexchange(connection, &channels_after_keyexchange);
+  
   connection->dispatch[SSH_MSG_GLOBAL_REQUEST]
     = &global_request_handler;
   connection->dispatch[SSH_MSG_CHANNEL_OPEN]
