@@ -2,7 +2,7 @@
  *
  * General channel type for forwarding data to an fd
  *
- * $Id: channel_forward.c,v 1.5 2001/12/05 12:21:18 nisse Exp $ */
+ * $Id: channel_forward.c,v 1.6 2002/03/14 15:35:08 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -59,7 +59,7 @@ init_channel_forward(struct channel_forward *self,
   
   self->socket = socket;
 
-  REMEMBER_RESOURCE(self->super.resources, &socket->super);
+  remember_resource(self->super.resources, &socket->super);
 }
 
 struct channel_forward *
