@@ -1,6 +1,6 @@
 /* client_session.c
  *
- * $Id: client_session.c,v 1.15 2001/10/30 17:00:29 nisse Exp $
+ * $Id: client_session.c,v 1.17 2002/03/20 13:38:55 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -168,9 +168,9 @@ do_client_io(struct command *s UNUSED,
     = make_channel_read_close_callback(channel);
 
   /* Make sure stdio is closed properly if the channel or connection dies */
-  REMEMBER_RESOURCE(channel->resources, &session->in->super);
-  REMEMBER_RESOURCE(channel->resources, &session->out->super);
-  REMEMBER_RESOURCE(channel->resources, &session->err->super);
+  remember_resource(channel->resources, &session->in->super);
+  remember_resource(channel->resources, &session->out->super);
+  remember_resource(channel->resources, &session->err->super);
   
   ALIST_SET(channel->request_types, ATOM_EXIT_STATUS,
 	    &make_handle_exit_status(session->exit_status)->super);
@@ -203,6 +203,13 @@ make_client_session_channel(struct lsh_fd *in,
    * dies, no matter when or how. */
   self->super.close = do_client_session_close;
 
+  /* We could miss the server's exit-status or exit-signal message if
+   * we close the channel directly at EOF. So don't do that.
+   *
+   * FIXME: Perhaps we need to set this bit again in do_exit_status
+   * and do_exit_signal. */
+  self->super.flags &= ~CHANNEL_CLOSE_AT_EOF;
+  
   /* FIXME: We make rec_window_size non-zero here, but we don't setup
    * the receive pointer until later, in do_client_io. That's bad. Do
    * something similar to server_session.c: Add an inital_window
@@ -220,9 +227,9 @@ make_client_session_channel(struct lsh_fd *in,
   self->err = err;
   self->escape = escape;
   
-  REMEMBER_RESOURCE(self->super.resources, &in->super);
-  REMEMBER_RESOURCE(self->super.resources, &out->super);
-  REMEMBER_RESOURCE(self->super.resources, &err->super);
+  remember_resource(self->super.resources, &in->super);
+  remember_resource(self->super.resources, &out->super);
+  remember_resource(self->super.resources, &err->super);
   
   /* Flow control */
   out->write_buffer->report = &self->super.super;
