@@ -1,6 +1,6 @@
 /* handshake.c
  *
- * $Id: handshake.c,v 1.6 2001/08/21 11:01:49 nisse Exp $
+ * $Id: handshake.c,v 1.12 2002/03/14 13:08:53 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -47,6 +47,52 @@
 
 #include "handshake.c.x"
 
+#if DATAFELLOWS_WORKAROUNDS
+/* Bug compatibility information. */
+struct compat_info
+{
+  const char *prefix;
+  enum peer_flag flags;
+};
+
+static const struct compat_info
+compat[] =
+  {
+    /* FIXME: Is there any 2.0.x without these bugs? */
+    { "2.0.", (PEER_SSH_DSS_KLUDGE | PEER_SERVICE_ACCEPT_KLUDGE
+	       | PEER_USERAUTH_REQUEST_KLUDGE | PEER_SEND_NO_DEBUG 
+	       | PEER_X11_OPEN_KLUDGE) },
+    { "2.1.0", (PEER_SSH_DSS_KLUDGE
+		| PEER_USERAUTH_REQUEST_KLUDGE | PEER_SEND_NO_DEBUG) },
+    { "3.0", PEER_SEND_NO_DEBUG },
+    { NULL, 0 }
+  };
+    
+static enum peer_flag
+compat_peer_flags(UINT32 length, UINT8 *software)
+{
+  unsigned i;
+  for (i = 0; compat[i].prefix; i++)
+    {
+      unsigned j;
+      for (j = 0; ; j++)
+	{
+	  if (!compat[i].prefix[j])
+	    /* Prefix matched */
+	    return compat[i].flags;
+
+	  else if (j == length || compat[i].prefix[j] != software[j])
+	    /* No match */
+	    break;
+	}
+    }
+  /* Default flags are 0 */
+  return 0;
+}
+#else /* !DATAFELLOWS_WORKAROUNDS */
+#define compat_peer_flags(a,b) 0
+#endif /* !DATAFELLOWS_WORKAROUNDS */
+
 /* GABA:
    (class
      (name connection_line_handler)
@@ -142,29 +188,12 @@ do_line(struct line_handler **h,
 		assert(mode == CONNECTION_SERVER);
 	      
 		/* Sending keyexchange packet was delayed. Do it now */
-		initiate_keyexchange(connection);
+		send_kexinit(connection);
 	      }
 #endif /* WITH_SSH1_FALLBACK */
 
-#if DATAFELLOWS_WORKAROUNDS
-	    if ( (swver_len >= 6) && !memcmp(swver, "2.0.", 4)
-		 /* FIXME: Perhaps do a numerical comparison here? */
-		 && (memcmp(swver + 4, "13", 2) <= 0) )
-	      {
-		connection->peer_flags
-		  |= (PEER_SSH_DSS_KLUDGE | PEER_SERVICE_ACCEPT_KLUDGE
-		      | PEER_USERAUTH_REQUEST_KLUDGE | PEER_SEND_NO_DEBUG 
-		      | PEER_X11_OPEN_KLUDGE);
-	      }
-	    else if ( (swver_len >= 5) && !memcmp(swver, "2.1.0", 5) )
-	      {
-		connection->peer_flags
-		  |= (PEER_SSH_DSS_KLUDGE
-		      | PEER_USERAUTH_REQUEST_KLUDGE | PEER_SEND_NO_DEBUG);
-		verbose("ssh.com 2.1.0\n");
-	      }
-#endif /* DATAFELLOWS_WORKAROUNDS */ 
-	    
+	    connection->peer_flags = compat_peer_flags(swver_len, swver);
+
 	    new = 
 	      make_read_packet(
 		make_packet_unpad(
@@ -256,7 +285,7 @@ make_connection_read_line(struct ssh_connection *connection,
 
 
 struct handshake_info *
-make_handshake_info(UINT32 flags,
+make_handshake_info(enum connection_flag flags,
 		    const char *id_comment,
 		    const char *debug_comment,
 		    UINT32 block_size,
@@ -282,19 +311,12 @@ make_handshake_info(UINT32 flags,
  * private keys (for the server) or a hostkey database (for the
  * client). */
 
-/* GABA:
-   (class
-     (name handshake_command_2)
-     (super command)
-     (vars
-       (info object handshake_info)
-       (init object make_kexinit)
-       (extra object lsh_object)))
-*/
-
 /* Buffer size when reading from the socket */
 #define BUF_SIZE (1<<14)
 
+/* Ten minutes */
+#define HANDSHAKE_TIMEOUT 600
+
 DEFINE_COMMAND4(handshake_command)
      (struct lsh_object *a1,
       struct lsh_object *a2,
@@ -355,8 +377,9 @@ DEFINE_COMMAND4(handshake_command)
   connection = make_ssh_connection
     (info->flags,
      lv->peer, info->debug_comment, 
-     c,
      make_exc_finish_read_handler(lv->fd, e, HANDLER_CONTEXT));
+
+  connection_after_keyexchange(connection, c);
   
   connection_init_io
     (connection, 
@@ -370,11 +393,15 @@ DEFINE_COMMAND4(handshake_command)
      ->write_buffer->super,
      info->random);
 
+  /* Install timeout. */
+  connection_set_timeout(connection,
+			 HANDSHAKE_TIMEOUT,
+			 "Handshake timed out");
+  
   connection->versions[mode] = version;
-  connection->kexinits[mode] = MAKE_KEXINIT(init); 
+  connection->kexinit = init; 
   connection->dispatch[SSH_MSG_KEXINIT]
-    = make_kexinit_handler(init,
-			   extra, info->algorithms);
+    = make_kexinit_handler(extra, info->algorithms);
 
 #if WITH_SSH1_FALLBACK
   /* In this mode the server SHOULD NOT send carriage return character (ascii
@@ -393,5 +420,5 @@ DEFINE_COMMAND4(handshake_command)
   A_WRITE(connection->raw,
 	  ssh_format("%lS\r\n", version));
   
-  initiate_keyexchange(connection);
+  send_kexinit(connection);
 }
