@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.156 2001/10/30 19:47:55 nisse Exp $ */
+ * $Id: io.c,v 1.162 2002/03/20 13:06:33 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -75,54 +75,7 @@
 /* Because of signal handlers, there can be only one oop object. */
 static oop_source_sys *global_oop_sys = NULL;
 static oop_source *source = NULL;
-
-void
-io_init(void)
-{
-  struct sigaction pipe;
-  memset(&pipe, 0, sizeof(pipe));
-
-  pipe.sa_handler = SIG_IGN;
-  sigemptyset(&pipe.sa_mask);
-  pipe.sa_flags = 0;
-  
-  if (sigaction(SIGPIPE, &pipe, NULL) < 0)
-    fatal("Failed to ignore SIGPIPE.\n");
-
-  assert(!global_oop_sys);
-  global_oop_sys = oop_sys_new();
-  if (!global_oop_sys)
-    fatal("Failed to initialize liboop.\n");
-
-  source = oop_sys_source(global_oop_sys);
-}
-
-void
-io_final(void)
-{
-  gc_final();
-  
-  /* There mustn't be any outstanding callbacks left. */
-  oop_sys_delete(global_oop_sys);
-  global_oop_sys = NULL;
-  source = NULL;
-}
-
-void
-io_run(void)
-{
-  void *res = oop_sys_run(global_oop_sys);
-
-  /* FIXME: OOP_ERROR is not defined in liboop-0.7. Upgrade, and then
-   * enable this check. */
-#if 0
-  if (res == OOP_ERROR)
-    werror("oop_sys_run (errno = %i):  %z\n",
-	   errno, STRERROR(errno));
-#endif
-  trace("io_run: Exiting\n");
-}
-
+static unsigned nfiles = 0;
 
 /* OOP Callbacks */
 static void *
@@ -146,8 +99,10 @@ lsh_oop_register_signal(struct lsh_signal_handler *handler)
   trace("lsh_oop_register_signal: signal: %i, handler: %t\n",
 	handler->signum, handler);
   
+  assert(source);
   if (handler->super.alive)
-    source->on_signal(source, handler->signum, lsh_oop_signal_callback, handler);
+    source->on_signal(source, handler->signum,
+		      lsh_oop_signal_callback, handler);
 }
 
 static void
@@ -156,12 +111,15 @@ lsh_oop_cancel_signal(struct lsh_signal_handler *handler)
   trace("lsh_oop_cancel_signal: signal: %i, handler: %t\n",
 	handler->signum, handler);
 
+  assert(source);
   if (handler->super.alive)
-    source->cancel_signal(source, handler->signum, lsh_oop_signal_callback, handler);
+    source->cancel_signal(source, handler->signum,
+			  lsh_oop_signal_callback, handler);
 }
 
 static void *
-lsh_oop_fd_read_callback(oop_source *s UNUSED, int fileno, oop_event event, void *data)
+lsh_oop_fd_read_callback(oop_source *s UNUSED, int fileno,
+			 oop_event event, void *data)
 {
   CAST(lsh_fd, fd, (struct lsh_object *) data);
 
@@ -183,6 +141,7 @@ lsh_oop_register_read_fd(struct lsh_fd *fd)
   trace("lsh_oop_register_read_fd: fd: %i, %z\n",
 	fd->fd, fd->label);
   
+  assert(source);
   if (fd->super.alive && !fd->want_read)
     {
       assert(fd->read);
@@ -198,6 +157,7 @@ lsh_oop_cancel_read_fd(struct lsh_fd *fd)
   trace("lsh_oop_cancel_read_fd: fd: %i, %z\n",
 	fd->fd, fd->label);
   
+  assert(source);
   if (fd->super.alive)
     {
       source->cancel_fd(source, fd->fd, OOP_READ);
@@ -206,7 +166,8 @@ lsh_oop_cancel_read_fd(struct lsh_fd *fd)
 }
 
 static void *
-lsh_oop_fd_write_callback(oop_source *s UNUSED, int fileno, oop_event event, void *data)
+lsh_oop_fd_write_callback(oop_source *s UNUSED, int fileno,
+			  oop_event event, void *data)
 {
   CAST(lsh_fd, fd, (struct lsh_object *) data);
 
@@ -228,6 +189,7 @@ lsh_oop_register_write_fd(struct lsh_fd *fd)
   trace("lsh_oop_register_write_fd: fd: %i, %z\n",
 	fd->fd, fd->label);
   
+  assert(source);
   if (fd->super.alive && !fd->want_write)
     {
       assert(fd->write);
@@ -243,6 +205,7 @@ lsh_oop_cancel_write_fd(struct lsh_fd *fd)
   trace("lsh_oop_cancel_write_fd: fd: %i, %z\n",
 	fd->fd, fd->label);
 
+  assert(source);
   if (fd->super.alive)
     {
       source->cancel_fd(source, fd->fd, OOP_WRITE);
@@ -271,15 +234,102 @@ lsh_oop_time_callback(oop_source *source UNUSED,
 static void
 lsh_oop_register_callout(struct lsh_callout *callout)
 {
+  assert(source);
+  trace("lsh_oop_register_callout: action: %t\n",
+        callout->action);
+
   if (callout->super.alive)
-    source->on_time(source, OOP_TIME_NOW, lsh_oop_time_callback, callout);
+    source->on_time(source, callout->when, lsh_oop_time_callback, callout);
 }
 
 static void
 lsh_oop_cancel_callout(struct lsh_callout *callout)
 {
+  assert(source);
+  trace("lsh_oop_cancel_callout: action: %t\n",
+        callout->action);
   if (callout->super.alive)
-    source->cancel_time(source, OOP_TIME_NOW, lsh_oop_time_callback, callout);
+    source->cancel_time(source, callout->when, lsh_oop_time_callback, callout);
+}
+
+static void *
+lsh_oop_stop_callback(oop_source *source UNUSED,
+                      struct timeval time UNUSED, void *data UNUSED)
+{
+  trace("lsh_oop_stop_callback\n");
+  
+  if (!nfiles)
+    /* An arbitrary non-NULL value stops oop_sys_run. */
+    return &nfiles;
+  else
+    return OOP_CONTINUE;
+}
+
+static void
+lsh_oop_stop(void)
+{
+  assert(source);
+  trace("lsh_oop_stop\n");
+  source->on_time(source, OOP_TIME_NOW, lsh_oop_stop_callback, NULL);
+}
+
+static void
+lsh_oop_cancel_stop(void)
+{
+  assert(source);
+  trace("lsh_oop_cancel_stop\n");
+  source->cancel_time(source, OOP_TIME_NOW, lsh_oop_stop_callback, NULL);
+}
+
+void
+io_init(void)
+{
+  struct sigaction pipe;
+  memset(&pipe, 0, sizeof(pipe));
+
+  pipe.sa_handler = SIG_IGN;
+  sigemptyset(&pipe.sa_mask);
+  pipe.sa_flags = 0;
+  
+  if (sigaction(SIGPIPE, &pipe, NULL) < 0)
+    fatal("Failed to ignore SIGPIPE.\n");
+
+  assert(!global_oop_sys);
+  global_oop_sys = oop_sys_new();
+  if (!global_oop_sys)
+    fatal("Failed to initialize liboop.\n");
+
+  source = oop_sys_source(global_oop_sys);
+}
+
+void
+io_final(void)
+{
+  assert(source);
+  gc_final();
+
+  /* The final gc may have closed some files, and called lsh_oop_stop.
+   * So we must unregister that before deleting the oop source. */
+  lsh_oop_cancel_stop();
+  
+  /* There mustn't be any outstanding callbacks left. */
+  oop_sys_delete(global_oop_sys);
+  global_oop_sys = NULL;
+  source = NULL;
+}
+
+void
+io_run(void)
+{
+  void *res = oop_sys_run(global_oop_sys);
+
+  /* We need liboop-0.8, OOP_ERROR is not defined in liboop-0.7. */
+
+  if (res == OOP_ERROR)
+    werror("oop_sys_run (errno = %i):  %z\n",
+	   errno, STRERROR(errno));
+
+  trace("io_run: Exiting\n");
 }
 
 
@@ -293,12 +343,12 @@ lsh_oop_cancel_callout(struct lsh_callout *callout)
        (action object lsh_callback)))
 */
 
-/* Scheduled calls. FIXME: For now, no scheduling in time. */
 /* GABA:
    (class
      (name lsh_callout)
      (super resource)
      (vars
+       (when . "struct timeval")
        (action object lsh_callback)))
 */
 
@@ -344,15 +394,25 @@ do_kill_callout(struct resource *s)
     }
 }
 
-/* Delays not implemented. */
 struct resource *
-io_callout(struct lsh_callback *action)
+io_callout(struct lsh_callback *action, unsigned seconds)
 {
   NEW(lsh_callout, self);
   init_resource(&self->super, do_kill_callout);
 
+  if (seconds)
+    {
+      /* NOTE: Using absolute times, like oop does, is a little
+       * dangerous if the system time is changed abruptly. */
+      if (gettimeofday(&self->when, NULL) < 0)
+	fatal("io_callout: gettimeofday failed!\n");
+      self->when.tv_sec += seconds;
+    }
+  else
+    self->when = OOP_TIME_NOW;
+  
   self->action = action;
-
+      
   lsh_oop_register_callout(self);
   
   gc_global(&self->super);
@@ -389,7 +449,7 @@ do_buffered_read(struct io_callback *s,
       case EPIPE:
 	/* Getting EPIPE from read seems strange, but appearantly
 	 * it happens sometimes. */
-	werror("Unexpected EPIPE.\n");
+	werror("Unexpected EPIPE from read.\n");
       default:
 	EXCEPTION_RAISE(fd->e, 
 			make_io_exception(EXC_IO_READ, fd,
@@ -1032,14 +1092,16 @@ address_info2sockaddr(socklen_t *length,
 		      const int *preference,
 		      int lookup)
 {
-  char *host;
+  const char *host;
 
   if (a->ip)
     {
-      host = alloca(a->ip->length + 1);
-  
-      memcpy(host, a->ip->data, a->ip->length);
-      host[a->ip->length] = '\0';
+      host = lsh_get_cstring(a->ip);
+      if (!host)
+	{
+	  debug("address_info2sockaddr: hostname contains NUL characters.\n");
+	  return NULL;
+	}
     }
   else
     host = NULL;
@@ -1055,7 +1117,7 @@ address_info2sockaddr(socklen_t *length,
     struct sockaddr *res;
     const int default_preference
 #if WITH_IPV6
-      [3] = { AF_INET6, AF_INET }
+      [3] = { AF_INET, AF_INET6, 0 }
 #else
       [2] = { AF_INET, 0 }
 #endif      
@@ -1216,6 +1278,7 @@ make_lsh_fd(int fd, const char *label,
 {
   NEW(lsh_fd, self);
 
+  nfiles++;
   io_init_fd(fd);
 
   init_resource(&self->super, do_kill_fd);
@@ -1237,6 +1300,12 @@ make_lsh_fd(int fd, const char *label,
   return self;
 }
 
+unsigned
+io_nfiles(void)
+{
+  return nfiles;
+}
+
 /* Some code is taken from Thomas Bellman's tcputils. */
 struct lsh_fd *
 io_connect(struct sockaddr *remote,
@@ -1660,6 +1729,9 @@ close_fd(struct lsh_fd *fd)
 			  make_io_exception(EXC_IO_CLOSE, fd,
 					    errno, NULL));
 	}
+      assert(nfiles);
+      if (!--nfiles)
+	lsh_oop_stop();
     }
   else
     werror("Closed already.\n");
