@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: io.h,v 1.86 2001/10/30 19:47:55 nisse Exp $ */
+ * $Id: io.h,v 1.88 2002/03/18 16:57:27 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -108,6 +108,9 @@
 #define FD_READ(fd) IO_CALLBACK((fd)->read, (fd))
 #define FD_WRITE(fd) IO_CALLBACK((fd)->write, (fd))
 
+/* Returns the number of open files. */
+unsigned
+io_nfiles(void);
 
 /* Used for read handlers like read_line and read_packet that
  * processes a little data at a time, possibly replacing the handler
@@ -234,7 +237,7 @@ io_signal_handler(int signum,
 		  struct lsh_callback *action);
 
 struct resource *
-io_callout(struct lsh_callback *action);
+io_callout(struct lsh_callback *action, unsigned seconds);
 
 int blocking_read(int fd, struct read_handler *r);
 
