@@ -1,6 +1,6 @@
 /* io_commands.c
  *
- * $Id: io_commands.c,v 1.49 2001/09/27 12:10:33 nisse Exp $
+ * $Id: io_commands.c,v 1.50 2002/03/14 15:35:08 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -212,7 +212,7 @@ do_connect(struct address_info *a,
     }
 
   if (resources)
-    REMEMBER_RESOURCE(resources,
+    remember_resource(resources,
 		      &fd->super);
 }
 
