@@ -2,7 +2,7 @@
  *
  * client main program
  *
- * $Id: lsh.c,v 1.156 2001/12/12 18:37:54 nisse Exp $ */
+ * $Id: lsh.c,v 1.158 2002/01/21 00:05:05 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -43,8 +43,8 @@
 #include "gateway_commands.h"
 #include "handshake.h"
 #include "lookup_verifier.h"
+#include "publickey_crypto.h"
 #include "randomness.h"
-#include "rsa.h"
 #include "sexp.h"
 #include "spki.h"
 #include "srp.h" 
@@ -359,6 +359,7 @@ read_user_keys(struct lsh_options *options)
                               -1);
       break;
 
+    case ATOM_RSA_PKCS1:
     case ATOM_RSA_PKCS1_SHA1:
       return make_object_list(2, 
                               make_keypair(ATOM_SSH_RSA,
@@ -413,7 +414,7 @@ STATIC_COMMAND(do_options2identities);
        ; If non-null, append an ACL for the received key to this file.
        (file object abstract_write)
        ; For fingerprinting
-       (hash object hash_algorithm)))
+       (hash const object hash_algorithm)))
 */
 
 static struct verifier *
@@ -584,7 +585,7 @@ make_lsh_host_db(struct spki_context *db,
   res->host = host;
   res->sloppy = sloppy;
   res->file = file;
-  res->hash = &sha1_algorithm;
+  res->hash = &crypto_sha1_algorithm;
 
   return &res->super;
 }
