@@ -1,6 +1,6 @@
 /* lsh.h
  *
- * $Id: lsh.h,v 1.23 2001/10/02 22:41:06 nisse Exp $ */
+ * $Id: lsh.h,v 1.25 2002/03/15 09:12:37 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -86,16 +86,13 @@ struct lsh_string_header
   struct lsh_string *next;
 };
 
-#else   /* !DEBUG_ALLOC */
-
-struct lsh_string_header {};
-
-#endif  /* !DEBUG_ALLOC */
+#endif  /* DEBUG_ALLOC */
 
 struct lsh_string
 {
+#if DEBUG_ALLOC
   struct lsh_string_header header;
-  
+#endif
   UINT32 sequence_number;
   /* NOTE: The allocated size may be larger than the string length. */
   UINT32 length; 
@@ -130,7 +127,7 @@ struct randomness;
 
 /* reaper.h */
 struct exit_callback;
-struct reap;
+struct reaper;
 
 /* resource.h */
 struct resource;
