@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: read_data.c,v 1.31 2000/06/10 13:23:41 nisse Exp $ */
+ * $Id: read_data.c,v 1.32 2002/03/14 12:56:50 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -60,6 +60,15 @@ do_read_data_query(struct io_consuming_read *s)
       return 0;
     }
 
+  /* If a keyexchange is in progress, we should stop reading. We rely
+   * on channels_after_keyexchange to restart reading. */
+  if (self->channel->connection->send_kex_only)
+    {
+      trace
+	("read_data: Data arrived during key exchange. Won't read it now.\n");
+      return 0;
+    }
+
   return MIN(self->channel->send_max_packet,
 	     self->channel->send_window_size);
 }
