@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.76 2001/12/05 12:35:47 nisse Exp $
+ * $Id: server_session.c,v 1.82 2002/03/16 11:31:54 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -226,8 +226,8 @@ struct lsh_string *
 format_exit_signal(struct ssh_channel *channel,
 		   int core, int signal)
 {
-  struct lsh_string *msg = ssh_format("Process killed by %lz.\n",
-				      STRSIGNAL(signal));
+  struct lsh_string *msg
+    = ssh_format("%lz.\n", STRSIGNAL(signal));
   
   return format_channel_request(ATOM_EXIT_SIGNAL,
 				channel,
@@ -337,6 +337,9 @@ make_pipes(int *in, int *out, int *err)
 
 #if WITH_PTY_SUPPORT
 
+/* Sets certain fd:s to -1, which means that the slave tty should be
+ * used (for the child), or that the stdout fd should be duplicated
+ * (for the parent). */
 static int
 make_pty(struct pty_info *pty, int *in, int *out, int *err)
 {
@@ -347,9 +350,8 @@ make_pty(struct pty_info *pty, int *in, int *out, int *err)
   debug("exists: \n"
         "  alive = %i\n"
         "  master = %i\n"
-        "  slave = %i\n"
         "... ",
-        pty->super.alive, pty->master, pty->slave);
+        pty->super.alive, pty->master);
   debug("\n");
   
   if (pty) 
@@ -358,9 +360,9 @@ make_pty(struct pty_info *pty, int *in, int *out, int *err)
       
       debug("make_pty: Using allocated pty.\n");
 
-      /* Ownership of the master fd:s is passed on to some file
-       * objects. We need an fd for window_change_request, but we have
-       * to use one of our regular fd:s to the master side, or we're
+      /* Ownership of the master fd is passed on to some file
+       * object. We need an fd for window_change_request, but we have
+       * to use our regular fd:s to the master side, or we're
        * disrupt EOF handling on either side. */
 
       pty->super.alive = 0;
@@ -369,57 +371,36 @@ make_pty(struct pty_info *pty, int *in, int *out, int *err)
        * could use a single lsh_fd object for the master side of the
        * pty. */
 
-      in[0] = pty->slave;
+      /* -1 means opening deferred to the child */
+      in[0] = -1;
       in[1] = pty->master;
-          
+      
       if ((out[0] = dup(pty->master)) < 0)
         {
           werror("make_pty: duping master pty to stdout failed (errno = %i): %z\n",
                  errno, STRERROR(errno));
 
-          close(in[0]);
-          close(in[1]);
-
           return 0;
         }
 
-      if ((out[1] = dup(pty->slave)) < 0)
-        {
-          werror("make_pty: duping slave pty to stdout failed (errno = %i): %z\n",
-                 errno, STRERROR(errno));
+      out[1] = -1;
 
-          close(in[0]);
-          close(in[1]);
-          close(out[0]);
+      /* pty_info no longer owns the pty fd */
+      pty->master = -1;
 
-          return 0;
-        }
 #if BASH_WORKAROUND
       /* Don't use a separate stderr channel; just dup the
        * stdout pty to stderr. */
-      if ((err[1] = dup(pty->slave)) < 0)
-        {
-          werror("make_pty: duping slave pty to stdout failed (errno = %i): %z\n",
-                 errno, STRERROR(errno));
-
-          close(in[0]);
-          close(in[1]);
-          close(out[0]);
-          close(out[1]);
-
-          return 0;
-        }
-      
+            
       err[0] = -1;
+      err[1] = -1;
       
 #else /* !BASH_WORKAROUND */
       if (!lsh_make_pipe(err))
         {
-          close(in[0]);
           close(in[1]);
           close(out[0]);
-          close(out[1]);
-
+	  
           return 0;
         }
 #endif /* !BASH_WORKAROUND */
@@ -434,196 +415,119 @@ static int make_pty(struct pty_info *pty UNUSED,
 { return 0; }
 #endif /* !WITH_PTY_SUPPORT */
 
-/* Returns -1 on failure, 0 for child and +1 for parent */
 static int
 spawn_process(struct server_session *session,
 	      struct lsh_user *user,
-	      struct address_info *peer)
+	      /* All information but the fd:s should be fileld in
+	       * already */
+	      struct spawn_info *info)
 {
-  int in[2];
-  int out[2];
-  int err[2];
-
-  if (session->process)
-    /* Already spawned a shell or command */
-    return -1;
+  struct exception_handler *io_exception_handler;
+  struct lsh_callback *read_close_callback;
   
-  /* {in|out|err}[0] is for reading,
-   * {in|out|err}[1] for writing. */
+  assert(!session->process);
 
-  if (session->pty && !make_pty(session->pty, in, out, err))
+  if (session->pty && !make_pty(session->pty,
+				info->in, info->out, info->err))
     {
+      KILL_RESOURCE(&session->pty->super);
       KILL(session->pty);
       session->pty = NULL;
     }
 
-  if (!session->pty && !make_pipes(in, out, err))
-    return -1;
+  if (!session->pty && !make_pipes(info->in, info->out, info->err))
+    return 0;
 
-  {
-    struct lsh_process *child;
-    
-    if (USER_FORK(user, &child,
-		  make_exit_shell(session),
-		  peer, session->pty ? session->pty->tty_name : NULL))
-      {
-	if (child)
-	  { /* Parent */
-	    struct ssh_channel *channel = &session->super;
-	    trace("spawn_process: Parent process\n");
-	    
-	    session->process = child;
+  session->process = USER_SPAWN(user, info, make_exit_shell(session));
 
-	    /* Close the child's fd:s */
-	    close(in[0]);
-	    close(out[1]);
-	    close(err[1]);
+  if (!session->process)
+    return 0;
+  
+  /* Exception handlers */
+  io_exception_handler
+    = make_channel_io_exception_handler(&session->super,
+					"lshd: Child stdio: ",
+					&default_exception_handler,
+					HANDLER_CONTEXT);
 
-	    {
-	      /* Exception handlers */
-	      struct exception_handler *io_exception_handler
-		= make_channel_io_exception_handler(channel,
-						    "lshd: Child stdio: ",
-						    &default_exception_handler,
-						    HANDLER_CONTEXT);
-
-	      /* Close callback for stderr and stdout */
-	      struct lsh_callback *read_close_callback
-		= make_channel_read_close_callback(channel);
-
-	      session->in
-		= io_write(make_lsh_fd(in[1], "child stdin",
-				       io_exception_handler),
-			   SSH_MAX_PACKET, NULL);
+  /* Close callback for stderr and stdout */
+  read_close_callback
+    = make_channel_read_close_callback(&session->super);
+  
+  session->in
+    = io_write(make_lsh_fd(info->in[1], "child stdin",
+			   io_exception_handler),
+	       SSH_MAX_PACKET, NULL);
 	  
-	      /* Flow control */
-	      session->in->write_buffer->report = &session->super.super;
+  /* Flow control */
+  session->in->write_buffer->report = &session->super.super;
+  
+  /* FIXME: Should we really use the same exception handler,
+   * which will close the channel on read errors, or is it
+   * better to just send EOF on read errors? */
+  session->out
+    = io_read(make_lsh_fd(info->out[0], "child stdout",
+			  io_exception_handler),
+	      make_channel_read_data(&session->super),
+	      read_close_callback);
+  session->err 
+    = ( (info->err[0] != -1)
+	? io_read(make_lsh_fd(info->err[0], "child stderr",
+			      io_exception_handler),
+		  make_channel_read_stderr(&session->super),
+		  read_close_callback)
+	: NULL);
 
-	      /* FIXME: Should we really use the same exception handler,
-	       * which will close the channel on read errors, or is it
-	       * better to just send EOF on read errors? */
-	      session->out
-		= io_read(make_lsh_fd(out[0], "child stdout",
-				      io_exception_handler),
-			  make_channel_read_data(channel),
-			  read_close_callback);
-	      session->err 
-		= ( (err[0] != -1)
-		    ? io_read(make_lsh_fd(err[0], "child stderr",
-					  io_exception_handler),
-			      make_channel_read_stderr(channel),
-			      read_close_callback)
-		    : NULL);
-	    }
-	
-	    channel->receive = do_receive;
-	    channel->send_adjust = do_send_adjust;
-	    channel->eof = do_eof;
+  session->super.receive = do_receive;
+  session->super.send_adjust = do_send_adjust;
+  session->super.eof = do_eof;
 	  
-	    /* Make sure that the process and it's stdio is
-	     * cleaned up if the channel or connection dies. */
-	    REMEMBER_RESOURCE
-	      (channel->resources, &child->super);
+  /* Make sure that the process and it's stdio is
+   * cleaned up if the channel or connection dies. */
+  remember_resource(session->super.resources, &session->process->super);
 
-	    /* FIXME: How to do this properly if in and out may use the
-	     * same fd? */
-	    REMEMBER_RESOURCE
-	      (channel->resources, &session->in->super);
-	    REMEMBER_RESOURCE
-	      (channel->resources, &session->out->super);
-	    if (session->err)
-	      REMEMBER_RESOURCE
-		(channel->resources, &session->err->super);
+  /* FIXME: How to do this properly if in and out may use the
+   * same fd? */
+  remember_resource(session->super.resources, &session->in->super);
+  remember_resource(session->super.resources, &session->out->super);
+  if (session->err)
+    remember_resource(session->super.resources, &session->err->super);
 
-	    /* Don't close channel immediately at EOF, as we want to
-	     * get a chance to send exit-status or exit-signal. */
-	    session->super.flags &= ~CHANNEL_CLOSE_AT_EOF;
-	    return 1;
-	  }
-	else
-	  { /* Child */
-	    trace("spawn_process: Child process\n");
-	    assert(getuid() == user->uid);
+  /* Don't close channel immediately at EOF, as we want to
+   * get a chance to send exit-status or exit-signal. */
+  session->super.flags &= ~CHANNEL_CLOSE_AT_EOF;
 
-#if 0
-	    /* Debug timing problems */
-	    if (sleep(5))
-	      {
-		trace("server_session.c: sleep interrupted\n");
+  channel_start_receive(&session->super, session->initial_window);
+  
+  return 1;
+}
 
-		sleep(5);
-	      }
-#endif    
-	    if (!USER_CHDIR_HOME(user))
-	      {
-		werror("Could not change to home (or root) directory!\n");
-		_exit(EXIT_FAILURE);
-	      }
-	    
-#if WITH_PTY_SUPPORT
-	    if (session->pty)
-	      {
-		debug("lshd: server.c: Setting controlling tty...\n");
-		if (!tty_setctty(session->pty))
-		  {
-		    debug("lshd: server.c: "
-			  "Setting controlling tty... Failed!\n");
-		    werror("lshd: Can't set controlling tty for child!\n");
-		    _exit(EXIT_FAILURE);
-		  }
-		else
-		  debug("lshd: server.c: Setting controlling tty... Ok.\n");
-	      }
-#endif /* WITH_PTY_SUPPORT */
-	  
-	    /* Close all descriptors but those used for communicationg
-	     * with parent. We rely on the close-on-exec flag for all
-	     * other fd:s. */
-	    
-	    if (dup2(in[0], STDIN_FILENO) < 0)
-	      {
-		werror("Can't dup stdin!\n");
-		_exit(EXIT_FAILURE);
-	      }
-	    close(in[0]);
-	    close(in[1]);
-	    
-	    if (dup2(out[1], STDOUT_FILENO) < 0)
-	      {
-		werror("Can't dup stdout!\n");
-		_exit(EXIT_FAILURE);
-	      }
-	    close(out[0]);
-	    close(out[1]);
+static void
+init_spawn_info(struct spawn_info *info, struct server_session *session,
+		unsigned argc, const char **argv,
+		unsigned env_length, struct env_value *env)
+{
+  unsigned i = 0;
+  
+  memset(info, 0, sizeof(*info));
+  info->peer = session->super.connection->peer;
+  info->pty = session->pty;
 
-	    if (!dup_error_stream())
-	      {
-		werror("server_session: Failed to dup old stderr. Bye.\n");
-		set_error_ignore();
-	      }
+  info->argc = argc;
+  info->argv = argv;
+  
+  assert(env_length >= 1);
 
-	    if (dup2(err[1], STDERR_FILENO) < 0)
-	      {
-		werror("Can't dup stderr!\n");
-		_exit(EXIT_FAILURE);
-	      }
-	    close(err[0]);
-	    close(err[1]);
-
-	    return 0;
-	  }
-      }
-    /* fork failed */
-    /* Close and return channel_failure */
-
-    close(err[0]);
-    close(err[1]);
-    close(out[0]);
-    close(out[1]);
-    close(in[0]);
-    close(in[1]);
-  }
-  return -1;
+  /* FIXME: Set SSH_TTY, SSH_CLIENT and SSH_ORIGINAL_COMMAND */
+  if (session->term)
+    {
+      env[i].name ="TERM";
+      env[i].value = session->term;
+      i++;
+    }
+  assert(i <= env_length);
+  info->env_length = i;
+  info->env = env;
 }
 
 DEFINE_CHANNEL_REQUEST(shell_request_handler)
@@ -635,8 +539,10 @@ DEFINE_CHANNEL_REQUEST(shell_request_handler)
       struct exception_handler *e)
 {
   CAST(server_session, session, channel);
-
-  static struct exception shell_request_failed =
+  struct spawn_info spawn;
+  struct env_value env;
+  
+  static const struct exception shell_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Shell request failed");
 
   if (!parse_eod(args))
@@ -649,63 +555,17 @@ DEFINE_CHANNEL_REQUEST(shell_request_handler)
     /* Already spawned a shell or command */
     goto fail;
 
-  switch (spawn_process(session, channel->connection->user,
-			channel->connection->peer))
+  init_spawn_info(&spawn, session, 0, NULL, 1, &env);
+  spawn.login = 1;
+    
+  if (spawn_process(session, channel->connection->user, &spawn))
+    /* NOTE: The return value is not used. */
+    COMMAND_RETURN(c, channel);
+  else
     {
-    case 1: /* Parent */
-      /* NOTE: The return value is not used. */
-      COMMAND_RETURN(c, channel);
-      channel_start_receive(channel, session->initial_window);
-      return;
-    case 0:
-      { /* Child */
-#define MAX_ENV 1
-	/* No args, and the USER_EXEC method fills in argv[0]. */
-	const char *argv[] = { NULL, NULL };
-	
-	struct env_value env[MAX_ENV];
-	int env_length = 0;
-	
-	debug("do_spawn_shell: Child process\n");
-	assert(getuid() == channel->connection->user->uid);
-	    	    
-	if (session->term)
-	  {
-	    env[env_length].name ="TERM";
-	    env[env_length].value = session->term;
-	    env_length++;
-	  }
-	assert(env_length <= MAX_ENV);
-#undef MAX_ENV
-
-#if 1
-	USER_EXEC(channel->connection->user, 1, argv, env_length, env);
-	
-	/* exec failed! */
-	verbose("server_session: exec failed (errno = %i): %z\n",
-		errno, STRERROR(errno));
-	_exit(EXIT_FAILURE);
-
-#else
-# define GREETING "Hello world!\n"
-	if (write(STDOUT_FILENO, GREETING, strlen(GREETING)) < 0)
-	  _exit(errno);
-	kill(getuid(), SIGSTOP);
-	if (write(STDOUT_FILENO, shell, strlen(shell)) < 0)
-	  _exit(125);
-	_exit(126);
-# undef GREETING
-#endif
-      }
-    case -1:
-      /* fork failed */
-
-      break;
-    default:
-      fatal("Internal error!");
-  }
- fail:
-  EXCEPTION_RAISE(e, &shell_request_failed);
+    fail:
+      EXCEPTION_RAISE(e, &shell_request_failed);
+    }
 }
 
 DEFINE_CHANNEL_REQUEST(exec_request_handler)
@@ -718,7 +578,7 @@ DEFINE_CHANNEL_REQUEST(exec_request_handler)
 {
   CAST(server_session, session, channel);
 
-  static struct exception exec_request_failed =
+  static const struct exception exec_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Exec request failed");
   
   UINT32 command_len;
@@ -739,67 +599,25 @@ DEFINE_CHANNEL_REQUEST(exec_request_handler)
     EXCEPTION_RAISE(e, &exec_request_failed);
   else
     {
-      struct lsh_string *command_line = ssh_format("%ls", command_len, command);
+      struct spawn_info spawn;
+      const char *args[2];
+      struct env_value env;
+
+      struct lsh_string *s = ssh_format("%ls", command_len, command);
       
-      switch (spawn_process(session, channel->connection->user,
-			    channel->connection->peer))
-	{
-	case 1: /* Parent */
-	  lsh_string_free(command_line);
-	  
-	  /* NOTE: The return value is not used. */
-	  COMMAND_RETURN(c, channel);
-	  channel_start_receive(channel, session->initial_window);
-	  return;
-	case 0:
-	  { /* Child */
-#define MAX_ENV 1
-	    struct env_value env[MAX_ENV];
-	    int env_length = 0;
-	    
-	    /* No args, and the USER_EXEC method fills in argv[0]. */
+      init_spawn_info(&spawn, session, 2, args, 1, &env);
+      spawn.login = 0;
+      
+      args[0] = "-c";
+      args[1] = lsh_get_cstring(s);
 
-	    /* NOTE: I'd like to use an array initializer, but that's
-	     * not ANSI-C, and at least HPUX' compiler can't handle
-	     * it. */
-	    
-	    const char *argv[4];
-	    argv[0] = NULL;
-	    argv[1] = "-c";
-	    argv[2] = lsh_get_cstring(command_line);
-	    argv[3] = NULL;
-	
-	    debug("do_spawn_shell: Child process\n");
+      if (spawn_process(session, channel->connection->user, &spawn))
+	/* NOTE: The return value is not used. */
+	COMMAND_RETURN(c, channel);
+      else
+	EXCEPTION_RAISE(e, &exec_request_failed);
 
-	    assert(getuid() == channel->connection->user->uid);
-	    assert(argv[2]);
-
-	    /* FIXME: Set SSH_TTY, SSH_CLIENT and SSH_ORIGINAL_COMMAND */
-	    if (session->term)
-	      {
-		env[env_length].name ="TERM";
-		env[env_length].value = session->term;
-		env_length++;
-	      }
-	    assert(env_length <= MAX_ENV);
-#undef MAX_ENV
-
-	    USER_EXEC(channel->connection->user, 0, argv, env_length, env);
-	
-	    /* exec failed! */
-	    verbose("server_session: exec failed (errno = %i): %z\n",
-		    errno, STRERROR(errno));
-	    _exit(EXIT_FAILURE);
-	  }
-	case -1:
-	  /* fork failed */
-	  lsh_string_free(command_line);
-	  EXCEPTION_RAISE(e, &exec_request_failed);
-
-	  break;
-	default:
-	  fatal("Internal error!");
-	}
+      lsh_string_free(s);
     }
 }
 
@@ -870,41 +688,27 @@ do_spawn_subsystem(struct channel_request *s,
   
   if (!session->process && program)
     {
+      struct spawn_info spawn;
+      const char *args[2];
+      struct env_value env;
+
       /* Don't use any pty */
       if (session->pty)
 	{
 	  KILL_RESOURCE(&session->pty->super);
 	  session->pty = NULL;
 	}
+
+      args[0] = "-c"; args[1] = program;
       
-      switch (spawn_process(session, channel->connection->user,
-			    channel->connection->peer))
+      init_spawn_info(&spawn, session, 2, args, 1, &env);
+      spawn.login = 0;
+
+      if (spawn_process(session, channel->connection->user, &spawn))
 	{
-	case 1: /* Parent */
-	  /* NOTE: The return value is not used. */
 	  COMMAND_RETURN(c, channel);
-	  channel_start_receive(channel, session->initial_window);
 	  return;
-
-	case 0: /* Child */
-	  {
-	    /* No args, and the USER_EXEC method fills in argv[0]. */
-	    const char *argv[] = { NULL, NULL };
-
-	    debug("do_spawn_subsystem: Child process\n");
-	  
-	    USER_EXEC(channel->connection->user, 1, argv, 0, NULL);
-
-	    werror("server_session: subsystem exec failed (errno = %i): %z\n",
-		   errno, STRERROR(errno));
-	    _exit(EXIT_FAILURE);
-	  }
-	case -1: /* Error */
-	  break;
-
-	default:
-	  fatal("Internal error!");
-	}
+	}  
     }
   EXCEPTION_RAISE(e, &subsystem_request_failed);
 }
@@ -932,84 +736,58 @@ do_alloc_pty(struct channel_request *c UNUSED,
 	     struct command_continuation *s,
 	     struct exception_handler *e)
 {
-  struct terminal_dimensions dims;
-  const UINT8 *mode;
-  UINT32 mode_length;
   struct lsh_string *term = NULL;
 
   static struct exception pty_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "pty request failed");
 
+  struct pty_info *pty = make_pty_info();
+  
   CAST(server_session, session, channel);
 
   verbose("Client requesting a tty...\n");
 
   if ((term = parse_string_copy(args))
-      && parse_uint32(args, &dims.char_width)
-      && parse_uint32(args, &dims.char_height)
-      && parse_uint32(args, &dims.pixel_width)
-      && parse_uint32(args, &dims.pixel_height)
-      && parse_string(args, &mode_length, &mode)
+      && parse_uint32(args, &pty->dims.char_width)
+      && parse_uint32(args, &pty->dims.char_height)
+      && parse_uint32(args, &pty->dims.pixel_width)
+      && parse_uint32(args, &pty->dims.pixel_height)
+      && (pty->mode = parse_string_copy(args))
       && parse_eod(args))
     {
       /* The client may only request one tty, and only before
        * starting a process. */
-      if (session->pty || session->process)
+      if (session->pty || session->process
+	  || !pty_open_master(pty, channel->connection->user->uid))
 	{
-	  lsh_string_free(term);
+	  verbose("Pty allocation failed.\n");
 	  EXCEPTION_RAISE(e, &pty_request_failed);
 	}
       else
 	{
-	  struct pty_info *pty = make_pty_info();
-      
-	  if (pty_allocate(pty, channel->connection->user->uid))
-	    {
-	      struct termios ios;
-	      
-	      if (tty_getattr(pty->slave, &ios))
-		{
-		  assert(pty->super.alive);
-		  session->pty = pty;
-		  
-		  /* Don't set TERM if the value is empty. */
-		  if (!term->length)
-		    {
-		      lsh_string_free(term);
-		      term = NULL;
-		    }
-	      
-		  session->term = term;
-		  tty_decode_term_mode(&ios, mode_length, mode); 
-	      
-		  if (tty_setattr(pty->slave, &ios) &&
-		      tty_setwinsize(pty->slave, &dims))
-		    {
-		      REMEMBER_RESOURCE(channel->resources, &pty->super);
+	  /* FIXME: Perhaps we can set the window dimensions directly
+	   * on the master pty? */
+	  session->term = term;
+	  session->pty = pty;
+	  remember_resource(channel->resources, &pty->super);
 
-		      verbose(" granted.\n");
-		      COMMAND_RETURN(s, NULL);
-		      
-		      return;
-		    }
-		}
-	    }
-	  
-	  /* Close fd:s and mark the pty-struct as dead */
-	  KILL_RESOURCE(&pty->super);
-	  KILL(pty);
+	  verbose(" granted.\n");
+	  COMMAND_RETURN(s, NULL);
+
+	  /* Success */
+	  return;
 	}
-      verbose("Pty allocation failed.\n");
-      lsh_string_free(term);
-      EXCEPTION_RAISE(e, &pty_request_failed);
+
     }
   else
     {
       werror("Invalid pty request.\n");
-      lsh_string_free(term);
-
       PROTOCOL_ERROR(e, "Invalid pty request.");
     }
+  /* Cleanup for failure cases. */
+  lsh_string_free(term);
+  KILL_RESOURCE(&pty->super);
+  KILL(pty);
 }
 
 struct channel_request
