@@ -1,6 +1,6 @@
 /* buffer.c
  *
- * $Id: buffer.c,v 1.9 2001/10/25 16:01:12 nisse Exp $
+ * $Id: buffer.c,v 1.11 2002/02/11 23:08:13 _cvs_pont Exp $
  *
  * Buffering for sftp.
  */
@@ -202,7 +202,25 @@ sftp_read_packet(struct sftp_input *i)
   int done;
 
   if (i->left)
-    return 0;
+    {
+      UINT8 d;
+
+      while (i->left &&                         /* Data remaining? */
+	     sftp_get_data(i, 1, &d)            /* Read OK? */
+	     )
+	;
+
+      /* Now, there shouldn't be any more data remaining. Next time
+       * we're called, the next packet should be read (or we had an
+       * error and all data is not read, if that's the case, return
+       * error).
+       */
+
+      if (i->left)    /* i->left non-zero => sftp_get_data returned zero => error  */
+	return -1; 
+
+      return 0;
+    }
 
   /* First, deallocate the strings. */
   sftp_input_clear_strings(i);
@@ -212,7 +230,12 @@ sftp_read_packet(struct sftp_input *i)
   switch (done)
     {
     case 0:
-      return feof(i->f) ? -1 : 0;
+      {
+	int err =  feof(i->f) ? -1 : 0;
+	clearerr( i->f );
+	return err;
+      }
+
     case 4:
       i->left = READ_UINT32(buf);
       return 1;
@@ -603,3 +626,9 @@ sftp_put_attrib(struct sftp_output *o, const struct sftp_attrib *a)
       sftp_put_uint32(o, a->mtime);
     }
 }
+
+int
+sftp_packet_size(struct sftp_output* out)
+{
+  return out->i;
+}
